# -*- coding: utf-8 -*-
"""
Telephonie  - Framework to create telephony applications using FreeSWITCH 

Primary Authors - Michael Ricordeau (tamiel) and Miglu (miglu)

Telephonie is inspired by and uses code from - https://github.com/fiorix/eventsocket
"""

__name__ = "telephonie"

__all__ = ['core', 'utils']

